/* This file defines the header fields of phy_radio that will be
 * added before the payload of each packet.
 * The upper layer (MAC) that use phy_radio should include this header
 * as its first field and CRC as its last field in each packet it 
 * declares (see smac_msg.h for example).
 * 
 * Authors: Wei Ye
*/

#ifndef PHY_MSG
#define PHY_MSG

// Maximum packet length -- including headers of all layers
// Each application can override the default max length in Makefile
// Maximum allowable value is 250
#ifndef PHY_MAX_PKT_LEN
#define PHY_MAX_PKT_LEN 100
#endif

// Physical-layer header to be put before data payload
typedef struct {
	uint8_t length; // length of entire packet
} PhyHeader;


// packet information to be recorded by physical layer
typedef struct {
	uint16_t strength;
	uint32_t timestamp;   // can be used w/ external counter of fine resolution
    uint32_t timeCoarse;  // S-MAC system time w/ resolution of 1 ms
} PhyPktInfo;

// Physical layer packet buffer (for receiving packets)
// Sending buffer should be provided by the top-level application

#define PHY_MIN_PKT_LEN (sizeof(PhyHeader) + 2)
#define PHY_MAX_PAYLOAD (PHY_MAX_PKT_LEN - PHY_MIN_PKT_LEN)

typedef struct {
	PhyHeader hdr;
	char data[PHY_MAX_PAYLOAD];
	int16_t crc;        // last field of a packet
	PhyPktInfo info;  // not part of a packet
} PhyPktBuf;

#endif  // PHY_MSG
