/* Defination of parameters and packet format for S-MAC
 * To be included by smac.c
 * If upper layer uses S-MAC it needs to include S-MAC header as the first
 * element in its own packet declaration.
 *
 * Author: Wei Ye (USC/ISI)
 */

#ifndef SMAC_MSG
#define SMAC_MSG

// In Berkeley's stack, it is defined in AM.h
// Since we are not useing it, define it here.
#ifndef TOS_BCAST_ADDR
#define TOS_BCAST_ADDR 0xffff
#endif

#ifndef PHY_MSG
#include "PhyRadioMsg.h"
#endif

// MAC header to be included by upper layer headers -- nested headers
typedef struct {
	PhyHeader phyHdr;
	char type;
	uint16_t toAddr;
	uint16_t fromAddr;
	uint16_t duration;
	uint8_t fragNo;
} MACHeader;


/************************************************************** 
This is an example showing how an application that used S-MAC to
to define its packet structures.

App-layer header should include MAC_Header as its first field, e.g.,

typedef struct {
	MACHeader hdr;
	// now add app-layer header fields
	char appField1;
	int16_t appField2;
} AppHeader;

This is an nested header structure, as MACHeader includes PhyHeader
as its first field.

You can get the maximum payload length by the following macro.

#define MAX_APP_PAYLOAD (PHY_MAX_PKT_LEN - sizeof(AppHeader) - 2)

The app packet with maximum allowed length is then

typedef struct {
	AppHeader hdr;
	char data[MAX_APP_PAYLOAD];
	int16_t crc;  // must be last two bytes, required by PhyRadio.
} AppPkt;

******************************************************************/

// control packet -- RTS, CTS, ACK
typedef struct {
	PhyHeader phyHdr;  // include before my own stuff
	char type;
	uint16_t toAddr;
	uint16_t fromAddr;
	uint16_t duration;
	int16_t crc;  // must be last two bytes, required by PhyRadio
} MACCtrlPkt;

// sync packet
typedef struct {
	PhyHeader phyHdr;  // include before my own stuff
	char type;
	uint16_t fromAddr;
	char state;
	uint8_t seqNo;
	uint16_t sleepTime;  // my next sleep time from now
	int16_t crc;  // must be last two bytes, required by PhyRadio
} MACSyncPkt;

// for performance measurement
typedef struct {
	uint32_t sleepTime;
	uint32_t idleTime;
	uint32_t rxTime;
	uint32_t txTime;
} RadioTime;

#endif //SMAC_MSG
